@@ -27,6 +27,8 @@ import os
 import re
 import src.xmlDefs
 import time
+import urllib
+import urlparse
 import uuid
 
 algorithms = {
@@ -146,6 +148,7 @@ class request(object):
         self.manager = manager
         self.host = self.manager.server_info.host
         self.port = self.manager.server_info.port
+        self.afunix = self.manager.server_info.afunix
         self.auth = True
         self.user = ""
         self.pswd = ""
@@ -227,16 +230,20 @@ class request(object):
 
     def gethttpdigestauth(self, si, wwwauthorize=None):
 
-        # Check the nonce cache to see if we've used this user before
+        # Check the nonce cache to see if we've used this user before, or if the nonce is more than 5 minutes old
         user = [self.user, si.user][self.user == ""]
         pswd = [self.pswd, si.pswd][self.pswd == ""]
         details = None
-        if user in self.manager.digestCache:
+        if user in self.manager.digestCache and self.manager.digestCache[user]["max-nonce-time"] > time.time():
             details = self.manager.digestCache[user]
         else:
-            http = SmartHTTPConnection(si.host, si.port, si.ssl)
+            # Redo digest auth from scratch to get a new nonce etc
+            http = SmartHTTPConnection(si.host, si.port, si.ssl, si.afunix)
             try:
-                http.request("OPTIONS", self.getURI(si))
+                puri = list(urlparse.urlparse(self.getURI(si)))
+                puri[2] = urllib.quote(puri[2])
+                quri = urlparse.urlunparse(puri)
+                http.request("OPTIONS", quri)
 
                 response = http.getresponse()
 
@@ -261,6 +268,7 @@ class request(object):
                     for (k, v) in [p.split('=', 1) for p in parts]:
                         details[k.strip()] = unq(v.strip())
 
+                    details["max-nonce-time"] = time.time() + 600
                     self.manager.digestCache[user] = details
                     break
 
@@ -381,6 +389,7 @@ class request(object):
         if node.get(src.xmlDefs.ATTR_HOST2, src.xmlDefs.ATTR_VALUE_NO) == src.xmlDefs.ATTR_VALUE_YES:
             self.host = self.manager.server_info.host2
             self.port = self.manager.server_info.port2
+            self.afunix = self.manager.server_info.afunix2
 
         for child in node.getchildren():
             if child.tag == src.xmlDefs.ELEMENT_REQUIRE_FEATURE:
